/*
 * Suggest module so user can suggest things for the zone
 *
 * 21/04/03 - Started by Smong. Reads settings from .conf and writes
 *  to specified file in the format: name,arena,suggestion with the
 *  the newest at the bottom. Exported global.conf:
 *
 *  [Suggest]
 *  file=suggest.txt
 *    ;file stored in data/
 *  floodtime=2000
 *    ;ticks between messages allowed from individual player
 *
 * 19/12/03 - Code fixes for 1.1.6. New anti-flood code based on
 *  a linked list and a timer.
 *
 * 13/02/04 - !text check. Suggestions now sent to mod chat too.
 *
 * 16/05/04 - Changed file format to 'time arena, name, suggestion'
 *  Time stamp code from log_file.c/nomysql.c
 *
 * 28/05/04 - renamed file from fb_suggest2.c to suggest.c
 *  Changed flood detection to use player data for cleanness.
 *
 */

#include <stdio.h>
#include "asss.h"

typedef struct
{
	ticks_t nextsuggestion;
} pdata;

local void append_time(FILE *f);
local void append_suggestion(Player *p, const char *text);

local void Csuggest(const char *text, Player *p, const Target *tgt);
local void PlayerAction(Player *p, int action, Arena *arena);

local int pdkey;

local Ilogman     *lm;
local Iconfig     *cfg;
local Iplayerdata *pd;
local Icmdman     *cmd;
local Ichat       *chat;

//Code from log_file.c
local void append_time(FILE *f)
{
	time_t t1;
	char t3[128];

	if (!f) return;

	time(&t1);
	strftime(t3, sizeof(t3), "%b %d %H:%M", localtime(&t1));
	fputs(t3, f);
}

local char default_file_name[] = { "suggest.txt" };

local void append_suggestion(Player *p, const char *text)
{
	FILE *f;
	const char *fname = NULL;
	char fullpath[48];

	if (cfg)
		fname = cfg->GetStr(GLOBAL, "Suggest", "file");
	if (!fname) fname = default_file_name;

	snprintf(fullpath, 42, "data/%s", fname);
	f = fopen(fullpath, "a");
	if (!f)
	{
		if (lm)
			lm->Log(L_DRIVEL, "<suggest> Could not open '%s' for writing", fname);
		return;
	}

	//Log time
	append_time(f);

	//Write to file
	fprintf(f, " %s,%s,%s\n", p->arena->name, p->name, text);
	chat->SendMessage(p, "Suggestion stored.");

	fclose(f);
}

local helptext_t suggest_help =
	"Args: <suggestion>\n"
	"Suggest new features for the zone or this arena.";

local void Csuggest(const char *text, Player *p, const Target *tgt)
{
	pdata *d = PPDATA(p, pdkey);
	ticks_t now = current_ticks();

	if (tgt->type != T_ARENA)
	{
		chat->SendMessage(p, "Send commands as public chat. Command ignored.");
		return;
	}

	if (!text || !p->arena) return;

	if (d->nextsuggestion > now)
	{
		/* ignore floods */
		if (lm)
			lm->LogP(L_MALICIOUS, "suggest", p, "flood detected");
		chat->SendMessage(p, "Processing last suggestion, please try again later.");
	}
	else
	{
		/* flood protection */
		int floodtime = 2000;
		if (cfg)
			floodtime = cfg->GetInt(GLOBAL, "Suggest","floodtime", 2000);
		d->nextsuggestion = now + floodtime;

		/* save suggestion */
		append_suggestion(p, text);
		chat->SendModMessage("suggest {%s} %s> %s", p->arena->name, p->name, text);
	}
}

local void PlayerAction(Player *p, int action, Arena *arena)
{
	pdata *d;

	if (action == PA_ENTERARENA)
	{
		d = PPDATA(p, pdkey);
		d->nextsuggestion = 0;
	}
}

EXPORT const char info_suggest[] = "v3";

EXPORT int MM_suggest(int action, Imodman *mm, Arena *arena)
{
	if (action == MM_LOAD)
	{
		lm   = mm->GetInterface(I_LOGMAN,     ALLARENAS);
		cfg  = mm->GetInterface(I_CONFIG,     ALLARENAS);
		pd   = mm->GetInterface(I_PLAYERDATA, ALLARENAS);
		cmd  = mm->GetInterface(I_CMDMAN,     ALLARENAS);
		chat = mm->GetInterface(I_CHAT,       ALLARENAS);
		if (!pd || !cmd || !chat)
			return MM_FAIL;

		pdkey = pd->AllocatePlayerData(sizeof(pdata));
		if (pdkey == -1) return MM_FAIL;

		mm->RegCallback(CB_PLAYERACTION, PlayerAction, ALLARENAS);
		cmd->AddCommand("suggest", Csuggest, suggest_help);

		return MM_OK;
	}
	else if (action == MM_UNLOAD)
	{
		cmd->RemoveCommand("suggest", Csuggest);
		mm->UnregCallback(CB_PLAYERACTION, PlayerAction, ALLARENAS);

		pd->FreePlayerData(pdkey);

		if (lm) mm->ReleaseInterface(lm);
		if (cfg) mm->ReleaseInterface(cfg);
		mm->ReleaseInterface(pd);
		mm->ReleaseInterface(cmd);
		mm->ReleaseInterface(chat);
		return MM_OK;
	}

	return MM_FAIL;
}
